# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.get_inventory_adjustment_response import GetInventoryAdjustmentResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.inventory_change import InventoryChangeParams
from ..types.batch_change_inventory_response import BatchChangeInventoryResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.inventory_change_type import InventoryChangeType
from ..types.inventory_state import InventoryState
from ..types.batch_get_inventory_changes_response import BatchGetInventoryChangesResponse
from ..types.batch_get_inventory_counts_response import BatchGetInventoryCountsResponse
from ..types.get_inventory_physical_count_response import GetInventoryPhysicalCountResponse
from ..types.get_inventory_transfer_response import GetInventoryTransferResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawInventoryClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def deprecated_get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetInventoryAdjustmentResponse]:
        """
        Deprecated version of [RetrieveInventoryAdjustment](api-endpoint:Inventory-RetrieveInventoryAdjustment) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetInventoryAdjustmentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/inventory/adjustment/{jsonable_encoder(adjustment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryAdjustmentResponse,
                    construct_type(
                        type_=GetInventoryAdjustmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetInventoryAdjustmentResponse]:
        """
        Returns the [InventoryAdjustment](entity:InventoryAdjustment) object
        with the provided `adjustment_id`.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetInventoryAdjustmentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/inventory/adjustments/{jsonable_encoder(adjustment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryAdjustmentResponse,
                    construct_type(
                        type_=GetInventoryAdjustmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deprecated_batch_change(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchChangeInventoryResponse]:
        """
        Deprecated version of [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchChangeInventoryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-change",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "changes": convert_and_respect_annotation_metadata(
                    object_=changes,
                    annotation=typing.Optional[typing.Sequence[InventoryChangeParams]],
                    direction="write",
                ),
                "ignore_unchanged_counts": ignore_unchanged_counts,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchChangeInventoryResponse,
                    construct_type(
                        type_=BatchChangeInventoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deprecated_batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchGetInventoryChangesResponse]:
        """
        Deprecated version of [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchGetInventoryChangesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-retrieve-changes",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "types": types,
                "states": states,
                "updated_after": updated_after,
                "updated_before": updated_before,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetInventoryChangesResponse,
                    construct_type(
                        type_=BatchGetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deprecated_batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchGetInventoryCountsResponse]:
        """
        Deprecated version of [BatchRetrieveInventoryCounts](api-endpoint:Inventory-BatchRetrieveInventoryCounts) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchGetInventoryCountsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-retrieve-counts",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "updated_after": updated_after,
                "cursor": cursor,
                "states": states,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetInventoryCountsResponse,
                    construct_type(
                        type_=BatchGetInventoryCountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_create_changes(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchChangeInventoryResponse]:
        """
        Applies adjustments and counts to the provided item quantities.

        On success: returns the current calculated counts for all objects
        referenced in the request.
        On failure: returns a list of related errors.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchChangeInventoryResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/inventory/changes/batch-create",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "changes": convert_and_respect_annotation_metadata(
                    object_=changes,
                    annotation=typing.Optional[typing.Sequence[InventoryChangeParams]],
                    direction="write",
                ),
                "ignore_unchanged_counts": ignore_unchanged_counts,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchChangeInventoryResponse,
                    construct_type(
                        type_=BatchChangeInventoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deprecated_get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetInventoryPhysicalCountResponse]:
        """
        Deprecated version of [RetrieveInventoryPhysicalCount](api-endpoint:Inventory-RetrieveInventoryPhysicalCount) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetInventoryPhysicalCountResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/inventory/physical-count/{jsonable_encoder(physical_count_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryPhysicalCountResponse,
                    construct_type(
                        type_=GetInventoryPhysicalCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetInventoryPhysicalCountResponse]:
        """
        Returns the [InventoryPhysicalCount](entity:InventoryPhysicalCount)
        object with the provided `physical_count_id`.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetInventoryPhysicalCountResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/inventory/physical-counts/{jsonable_encoder(physical_count_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryPhysicalCountResponse,
                    construct_type(
                        type_=GetInventoryPhysicalCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_transfer(
        self, transfer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetInventoryTransferResponse]:
        """
        Returns the [InventoryTransfer](entity:InventoryTransfer) object
        with the provided `transfer_id`.

        Parameters
        ----------
        transfer_id : str
            ID of the [InventoryTransfer](entity:InventoryTransfer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetInventoryTransferResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/inventory/transfers/{jsonable_encoder(transfer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryTransferResponse,
                    construct_type(
                        type_=GetInventoryTransferResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawInventoryClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def deprecated_get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetInventoryAdjustmentResponse]:
        """
        Deprecated version of [RetrieveInventoryAdjustment](api-endpoint:Inventory-RetrieveInventoryAdjustment) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetInventoryAdjustmentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/inventory/adjustment/{jsonable_encoder(adjustment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryAdjustmentResponse,
                    construct_type(
                        type_=GetInventoryAdjustmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_adjustment(
        self, adjustment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetInventoryAdjustmentResponse]:
        """
        Returns the [InventoryAdjustment](entity:InventoryAdjustment) object
        with the provided `adjustment_id`.

        Parameters
        ----------
        adjustment_id : str
            ID of the [InventoryAdjustment](entity:InventoryAdjustment) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetInventoryAdjustmentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/inventory/adjustments/{jsonable_encoder(adjustment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryAdjustmentResponse,
                    construct_type(
                        type_=GetInventoryAdjustmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deprecated_batch_change(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchChangeInventoryResponse]:
        """
        Deprecated version of [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchChangeInventoryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-change",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "changes": convert_and_respect_annotation_metadata(
                    object_=changes,
                    annotation=typing.Optional[typing.Sequence[InventoryChangeParams]],
                    direction="write",
                ),
                "ignore_unchanged_counts": ignore_unchanged_counts,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchChangeInventoryResponse,
                    construct_type(
                        type_=BatchChangeInventoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deprecated_batch_get_changes(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        types: typing.Optional[typing.Sequence[InventoryChangeType]] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        updated_before: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchGetInventoryChangesResponse]:
        """
        Deprecated version of [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is only applicable when set. The default value is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            The filter is only applicable when set. The default value is null.

        types : typing.Optional[typing.Sequence[InventoryChangeType]]
            The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
            The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return `ADJUSTMENT` query results by
            `InventoryState`. This filter is only applied when set.
            The default value is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        updated_before : typing.Optional[str]
            The filter to return results with their `created_at` or `calculated_at` value
            strictly before the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryChange) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchGetInventoryChangesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-retrieve-changes",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "types": types,
                "states": states,
                "updated_after": updated_after,
                "updated_before": updated_before,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetInventoryChangesResponse,
                    construct_type(
                        type_=BatchGetInventoryChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deprecated_batch_get_counts(
        self,
        *,
        catalog_object_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        updated_after: typing.Optional[str] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        states: typing.Optional[typing.Sequence[InventoryState]] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchGetInventoryCountsResponse]:
        """
        Deprecated version of [BatchRetrieveInventoryCounts](api-endpoint:Inventory-BatchRetrieveInventoryCounts) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        catalog_object_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `CatalogObject` ID.
            The filter is applicable only when set.  The default is null.

        location_ids : typing.Optional[typing.Sequence[str]]
            The filter to return results by `Location` ID.
            This filter is applicable only when set. The default is null.

        updated_after : typing.Optional[str]
            The filter to return results with their `calculated_at` value
            after the given time as specified in an RFC 3339 timestamp.
            The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for the original query.

            See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.

        states : typing.Optional[typing.Sequence[InventoryState]]
            The filter to return results by `InventoryState`. The filter is only applicable when set.
            Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
            The default is null.

        limit : typing.Optional[int]
            The number of [records](entity:InventoryCount) to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchGetInventoryCountsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/inventory/batch-retrieve-counts",
            method="POST",
            json={
                "catalog_object_ids": catalog_object_ids,
                "location_ids": location_ids,
                "updated_after": updated_after,
                "cursor": cursor,
                "states": states,
                "limit": limit,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetInventoryCountsResponse,
                    construct_type(
                        type_=BatchGetInventoryCountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_create_changes(
        self,
        *,
        idempotency_key: str,
        changes: typing.Optional[typing.Sequence[InventoryChangeParams]] = OMIT,
        ignore_unchanged_counts: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchChangeInventoryResponse]:
        """
        Applies adjustments and counts to the provided item quantities.

        On success: returns the current calculated counts for all objects
        referenced in the request.
        On failure: returns a list of related errors.

        Parameters
        ----------
        idempotency_key : str
            A client-supplied, universally unique identifier (UUID) for the
            request.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
            [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
            information.

        changes : typing.Optional[typing.Sequence[InventoryChangeParams]]
            The set of physical counts and inventory adjustments to be made.
            Changes are applied based on the client-supplied timestamp and may be sent
            out of order.

        ignore_unchanged_counts : typing.Optional[bool]
            Indicates whether the current physical count should be ignored if
            the quantity is unchanged since the last physical count. Default: `true`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchChangeInventoryResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/inventory/changes/batch-create",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "changes": convert_and_respect_annotation_metadata(
                    object_=changes,
                    annotation=typing.Optional[typing.Sequence[InventoryChangeParams]],
                    direction="write",
                ),
                "ignore_unchanged_counts": ignore_unchanged_counts,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchChangeInventoryResponse,
                    construct_type(
                        type_=BatchChangeInventoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deprecated_get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetInventoryPhysicalCountResponse]:
        """
        Deprecated version of [RetrieveInventoryPhysicalCount](api-endpoint:Inventory-RetrieveInventoryPhysicalCount) after the endpoint URL
        is updated to conform to the standard convention.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetInventoryPhysicalCountResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/inventory/physical-count/{jsonable_encoder(physical_count_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryPhysicalCountResponse,
                    construct_type(
                        type_=GetInventoryPhysicalCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_physical_count(
        self, physical_count_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetInventoryPhysicalCountResponse]:
        """
        Returns the [InventoryPhysicalCount](entity:InventoryPhysicalCount)
        object with the provided `physical_count_id`.

        Parameters
        ----------
        physical_count_id : str
            ID of the
            [InventoryPhysicalCount](entity:InventoryPhysicalCount) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetInventoryPhysicalCountResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/inventory/physical-counts/{jsonable_encoder(physical_count_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryPhysicalCountResponse,
                    construct_type(
                        type_=GetInventoryPhysicalCountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_transfer(
        self, transfer_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetInventoryTransferResponse]:
        """
        Returns the [InventoryTransfer](entity:InventoryTransfer) object
        with the provided `transfer_id`.

        Parameters
        ----------
        transfer_id : str
            ID of the [InventoryTransfer](entity:InventoryTransfer) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetInventoryTransferResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/inventory/transfers/{jsonable_encoder(transfer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetInventoryTransferResponse,
                    construct_type(
                        type_=GetInventoryTransferResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .gift_card_activity import GiftCardActivityParams


class CreateGiftCardActivityResponseParams(typing_extensions.TypedDict):
    """
    A response that contains a `GiftCardActivity` that was created.
    The response might contain a set of `Error` objects if the request resulted in errors.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    gift_card_activity: typing_extensions.NotRequired[GiftCardActivityParams]
    """
    The gift card activity that was created.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class FulfillmentFulfillmentEntry(UncheckedBaseModel):
    """
    Links an order line item to a fulfillment. Each entry must reference
    a valid `uid` for an order line item in the `line_item_uid` field, as well as a `quantity` to
    fulfill.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the fulfillment entry only within this order.
    """

    line_item_uid: str = pydantic.Field()
    """
    The `uid` from the order line item.
    """

    quantity: str = pydantic.Field()
    """
    The quantity of the line item being fulfilled, formatted as a decimal number.
    For example, `"3"`.
    
    Fulfillments for line items with a `quantity_unit` can have non-integer quantities.
    For example, `"1.70000"`.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Application-defined data attached to this fulfillment entry. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .booking_creator_details_creator_type import BookingCreatorDetailsCreatorType
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BookingCreatorDetails(UncheckedBaseModel):
    """
    Information about a booking creator.
    """

    creator_type: typing.Optional[BookingCreatorDetailsCreatorType] = pydantic.Field(default=None)
    """
    The seller-accessible type of the creator of the booking.
    See [BookingCreatorDetailsCreatorType](#type-bookingcreatordetailscreatortype) for possible values
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the team member who created the booking, when the booking creator is of the `TEAM_MEMBER` type.
    Access to this field requires seller-level permissions.
    """

    customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the customer who created the booking, when the booking creator is of the `CUSTOMER` type.
    Access to this field requires seller-level permissions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .job_assignment import JobAssignmentParams


class WageSettingParams(typing_extensions.TypedDict):
    """
    Represents information about the overtime exemption status, job assignments, and compensation
    for a [team member](entity:TeamMember).
    """

    team_member_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the team member associated with the wage setting.
    """

    job_assignments: typing_extensions.NotRequired[typing.Optional[typing.Sequence[JobAssignmentParams]]]
    """
    **Required** The ordered list of jobs that the team member is assigned to.
    The first job assignment is considered the team member's primary job.
    """

    is_overtime_exempt: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether the team member is exempt from the overtime rules of the seller's country.
    """

    version: typing_extensions.NotRequired[int]
    """
    **Read only** Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If not provided,
    Square executes a blind write, potentially overwriting data from another write. For more information,
    see [optimistic concurrency](https://developer.squareup.com/docs/working-with-apis/optimistic-concurrency).
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the wage setting was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the wage setting was last updated, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .catalog_object import CatalogObjectParams


class CreateCatalogImageRequestParams(typing_extensions.TypedDict):
    idempotency_key: str
    """
    A unique string that identifies this CreateCatalogImage request.
    Keys can be any valid string but must be unique for every CreateCatalogImage request.
    
    See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.
    """

    object_id: typing_extensions.NotRequired[str]
    """
    Unique ID of the `CatalogObject` to attach this `CatalogImage` object to. Leave this
    field empty to create unattached images, for example if you are building an integration
    where an image can be attached to catalog items at a later time.
    """

    image: CatalogObjectParams
    """
    The new `CatalogObject` of the `IMAGE` type, namely, a `CatalogImage` object, to encapsulate the specified image file.
    """

    is_primary: typing_extensions.NotRequired[bool]
    """
    If this is set to `true`, the image created will be the primary, or first image of the object referenced by `object_id`.
    If the `CatalogObject` already has a primary `CatalogImage`, setting this field to `true` will replace the primary image.
    If this is set to `false` and you use the Square API version 2021-12-15 or later, the image id will be appended to the list of `image_ids` on the object.
    
    With Square API version 2021-12-15 or later, the default value is `false`. Otherwise, the effective default value is `true`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.bulk_delete_merchant_custom_attributes_request_merchant_custom_attribute_delete_request import (
    BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams,
)
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.bulk_delete_merchant_custom_attributes_response import BulkDeleteMerchantCustomAttributesResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.bulk_upsert_merchant_custom_attributes_request_merchant_custom_attribute_upsert_request import (
    BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams,
)
from ...types.bulk_upsert_merchant_custom_attributes_response import BulkUpsertMerchantCustomAttributesResponse
from ...types.retrieve_merchant_custom_attribute_response import RetrieveMerchantCustomAttributeResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...requests.custom_attribute import CustomAttributeParams
from ...types.upsert_merchant_custom_attribute_response import UpsertMerchantCustomAttributeResponse
from ...types.delete_merchant_custom_attribute_response import DeleteMerchantCustomAttributeResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkDeleteMerchantCustomAttributesResponse]:
        """
        Deletes [custom attributes](entity:CustomAttribute) for a merchant as a bulk operation.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams]
            The data used to update the `CustomAttribute` objects.
            The keys must be unique and are used to map to the corresponding response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkDeleteMerchantCustomAttributesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/merchants/custom-attributes/bulk-delete",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[
                        str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams
                    ],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkDeleteMerchantCustomAttributesResponse,
                    construct_type(
                        type_=BulkDeleteMerchantCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BulkUpsertMerchantCustomAttributesResponse]:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for a merchant as a bulk operation.
        Use this endpoint to set the value of one or more custom attributes for a merchant.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateMerchantCustomAttributeDefinition](api-endpoint:MerchantCustomAttributes-CreateMerchantCustomAttributeDefinition) endpoint.
        This `BulkUpsertMerchantCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a merchant ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertMerchantCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkUpsertMerchantCustomAttributesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/merchants/custom-attributes/bulk-upsert",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[
                        str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams
                    ],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpsertMerchantCustomAttributesResponse,
                    construct_type(
                        type_=BulkUpsertMerchantCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        merchant_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RetrieveMerchantCustomAttributeResponse]:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with a merchant.
        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.
        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveMerchantCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="GET",
            params={
                "with_definition": with_definition,
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveMerchantCustomAttributeResponse,
                    construct_type(
                        type_=RetrieveMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert(
        self,
        merchant_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpsertMerchantCustomAttributeResponse]:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for a merchant.
        Use this endpoint to set the value of a custom attribute for a specified merchant.
        A custom attribute is based on a custom attribute definition in a Square seller account, which
        is created using the [CreateMerchantCustomAttributeDefinition](api-endpoint:MerchantCustomAttributes-CreateMerchantCustomAttributeDefinition) endpoint.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:
            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - The version field must match the current version of the custom attribute definition to enable
            [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            If this is not important for your application, version can be set to -1. For any other values, the request fails with a BAD_REQUEST error.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpsertMerchantCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="POST",
            json={
                "custom_attribute": convert_and_respect_annotation_metadata(
                    object_=custom_attribute, annotation=CustomAttributeParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertMerchantCustomAttributeResponse,
                    construct_type(
                        type_=UpsertMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, merchant_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteMerchantCustomAttributeResponse]:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a merchant.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteMerchantCustomAttributeResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteMerchantCustomAttributeResponse,
                    construct_type(
                        type_=DeleteMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCustomAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkDeleteMerchantCustomAttributesResponse]:
        """
        Deletes [custom attributes](entity:CustomAttribute) for a merchant as a bulk operation.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams]
            The data used to update the `CustomAttribute` objects.
            The keys must be unique and are used to map to the corresponding response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkDeleteMerchantCustomAttributesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/merchants/custom-attributes/bulk-delete",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[
                        str, BulkDeleteMerchantCustomAttributesRequestMerchantCustomAttributeDeleteRequestParams
                    ],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkDeleteMerchantCustomAttributesResponse,
                    construct_type(
                        type_=BulkDeleteMerchantCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BulkUpsertMerchantCustomAttributesResponse]:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for a merchant as a bulk operation.
        Use this endpoint to set the value of one or more custom attributes for a merchant.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateMerchantCustomAttributeDefinition](api-endpoint:MerchantCustomAttributes-CreateMerchantCustomAttributeDefinition) endpoint.
        This `BulkUpsertMerchantCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a merchant ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertMerchantCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkUpsertMerchantCustomAttributesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/merchants/custom-attributes/bulk-upsert",
            method="POST",
            json={
                "values": convert_and_respect_annotation_metadata(
                    object_=values,
                    annotation=typing.Dict[
                        str, BulkUpsertMerchantCustomAttributesRequestMerchantCustomAttributeUpsertRequestParams
                    ],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkUpsertMerchantCustomAttributesResponse,
                    construct_type(
                        type_=BulkUpsertMerchantCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        merchant_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RetrieveMerchantCustomAttributeResponse]:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with a merchant.
        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.
        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveMerchantCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="GET",
            params={
                "with_definition": with_definition,
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveMerchantCustomAttributeResponse,
                    construct_type(
                        type_=RetrieveMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert(
        self,
        merchant_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpsertMerchantCustomAttributeResponse]:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for a merchant.
        Use this endpoint to set the value of a custom attribute for a specified merchant.
        A custom attribute is based on a custom attribute definition in a Square seller account, which
        is created using the [CreateMerchantCustomAttributeDefinition](api-endpoint:MerchantCustomAttributes-CreateMerchantCustomAttributeDefinition) endpoint.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:
            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - The version field must match the current version of the custom attribute definition to enable
            [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            If this is not important for your application, version can be set to -1. For any other values, the request fails with a BAD_REQUEST error.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpsertMerchantCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="POST",
            json={
                "custom_attribute": convert_and_respect_annotation_metadata(
                    object_=custom_attribute, annotation=CustomAttributeParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpsertMerchantCustomAttributeResponse,
                    construct_type(
                        type_=UpsertMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, merchant_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteMerchantCustomAttributeResponse]:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a merchant.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        merchant_id : str
            The ID of the target [merchant](entity:Merchant).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteMerchantCustomAttributeResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/merchants/{jsonable_encoder(merchant_id)}/custom-attributes/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteMerchantCustomAttributeResponse,
                    construct_type(
                        type_=DeleteMerchantCustomAttributeResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogObjectReference(UncheckedBaseModel):
    """
    A reference to a Catalog object at a specific version. In general this is
    used as an entry point into a graph of catalog objects, where the objects exist
    at a specific version.
    """

    object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the referenced object.
    """

    catalog_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the object.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .standard_unit_description import StandardUnitDescriptionParams


class StandardUnitDescriptionGroupParams(typing_extensions.TypedDict):
    """
    Group of standard measurement units.
    """

    standard_unit_descriptions: typing_extensions.NotRequired[
        typing.Optional[typing.Sequence[StandardUnitDescriptionParams]]
    ]
    """
    List of standard (non-custom) measurement units in this description group.
    """

    language_code: typing_extensions.NotRequired[typing.Optional[str]]
    """
    IETF language tag.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.create_catalog_image_request import CreateCatalogImageRequestParams
from ... import core
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_catalog_image_response import CreateCatalogImageResponse
import json
from ...core.jsonable_encoder import jsonable_encoder
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.update_catalog_image_request import UpdateCatalogImageRequestParams
from ...types.update_catalog_image_response import UpdateCatalogImageResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawImagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        request: typing.Optional[CreateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCatalogImageResponse]:
        """
        Uploads an image file to be represented by a [CatalogImage](entity:CatalogImage) object that can be linked to an existing
        [CatalogObject](entity:CatalogObject) instance. The resulting `CatalogImage` is unattached to any `CatalogObject` if the `object_id`
        is not specified.

        This `CreateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        request : typing.Optional[CreateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCatalogImageResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/images",
            method="POST",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCatalogImageResponse,
                    construct_type(
                        type_=CreateCatalogImageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        image_id: str,
        *,
        request: typing.Optional[UpdateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateCatalogImageResponse]:
        """
        Uploads a new image file to replace the existing one in the specified [CatalogImage](entity:CatalogImage) object.

        This `UpdateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        image_id : str
            The ID of the `CatalogImage` object to update the encapsulated image file.

        request : typing.Optional[UpdateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateCatalogImageResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/catalog/images/{jsonable_encoder(image_id)}",
            method="PUT",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCatalogImageResponse,
                    construct_type(
                        type_=UpdateCatalogImageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawImagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        request: typing.Optional[CreateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCatalogImageResponse]:
        """
        Uploads an image file to be represented by a [CatalogImage](entity:CatalogImage) object that can be linked to an existing
        [CatalogObject](entity:CatalogObject) instance. The resulting `CatalogImage` is unattached to any `CatalogObject` if the `object_id`
        is not specified.

        This `CreateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        request : typing.Optional[CreateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCatalogImageResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/images",
            method="POST",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCatalogImageResponse,
                    construct_type(
                        type_=CreateCatalogImageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        image_id: str,
        *,
        request: typing.Optional[UpdateCatalogImageRequestParams] = OMIT,
        image_file: typing.Optional[core.File] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateCatalogImageResponse]:
        """
        Uploads a new image file to replace the existing one in the specified [CatalogImage](entity:CatalogImage) object.

        This `UpdateCatalogImage` endpoint accepts HTTP multipart/form-data requests with a JSON part and an image file part in
        JPEG, PJPEG, PNG, or GIF format. The maximum file size is 15MB.

        Parameters
        ----------
        image_id : str
            The ID of the `CatalogImage` object to update the encapsulated image file.

        request : typing.Optional[UpdateCatalogImageRequestParams]

        image_file : typing.Optional[core.File]
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateCatalogImageResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/catalog/images/{jsonable_encoder(image_id)}",
            method="PUT",
            data={},
            files={
                **(
                    {"request": (None, json.dumps(jsonable_encoder(request)), "application/json; charset=utf-8")}
                    if request is not OMIT
                    else {}
                ),
                **(
                    {"image_file": core.with_content_type(file=image_file, default_content_type="image/jpeg")}
                    if image_file is not None
                    else {}
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCatalogImageResponse,
                    construct_type(
                        type_=UpdateCatalogImageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .invoice_custom_field_placement import InvoiceCustomFieldPlacement
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceCustomField(UncheckedBaseModel):
    """
    An additional seller-defined and customer-facing field to include on the invoice. For more information,
    see [Custom fields](https://developer.squareup.com/docs/invoices-api/overview#custom-fields).

    Adding custom fields to an invoice requires an
    [Invoices Plus subscription](https://developer.squareup.com/docs/invoices-api/overview#invoices-plus-subscription).
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    The label or title of the custom field. This field is required for a custom field.
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The text of the custom field. If omitted, only the label is rendered.
    """

    placement: typing.Optional[InvoiceCustomFieldPlacement] = pydantic.Field(default=None)
    """
    The location of the custom field on the invoice. This field is required for a custom field.
    See [InvoiceCustomFieldPlacement](#type-invoicecustomfieldplacement) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawSegmentsClient
import typing
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.customer_segment import CustomerSegment
from ...types.list_customer_segments_response import ListCustomerSegmentsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_customer_segment_response import GetCustomerSegmentResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawSegmentsClient
from ...core.pagination import AsyncPager


class SegmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSegmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSegmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSegmentsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomerSegment]:
        """
        Retrieves the list of customer segments of a business.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by previous calls to `ListCustomerSegments`.
            This cursor is used to retrieve the next set of query results.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is less than 1 or greater than 50, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 50.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomerSegment]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.customers.segments.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/segments",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerSegmentsResponse,
                    construct_type(
                        type_=ListCustomerSegmentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.segments
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, segment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerSegmentResponse:
        """
        Retrieves a specific customer segment as identified by the `segment_id` value.

        Parameters
        ----------
        segment_id : str
            The Square-issued ID of the customer segment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerSegmentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.segments.get(
            segment_id="segment_id",
        )
        """
        response = self._raw_client.get(segment_id, request_options=request_options)
        return response.data


class AsyncSegmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSegmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSegmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSegmentsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomerSegment]:
        """
        Retrieves the list of customer segments of a business.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by previous calls to `ListCustomerSegments`.
            This cursor is used to retrieve the next set of query results.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the specified limit is less than 1 or greater than 50, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 50.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomerSegment]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.customers.segments.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/segments",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerSegmentsResponse,
                    construct_type(
                        type_=ListCustomerSegmentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.segments
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, segment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerSegmentResponse:
        """
        Retrieves a specific customer segment as identified by the `segment_id` value.

        Parameters
        ----------
        segment_id : str
            The Square-issued ID of the customer segment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerSegmentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.segments.get(
                segment_id="segment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(segment_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class GiftCardActivityTransferBalanceTo(UncheckedBaseModel):
    """
    Represents details about a `TRANSFER_BALANCE_TO` [gift card activity type](entity:GiftCardActivityType).
    """

    transfer_from_gift_card_id: str = pydantic.Field()
    """
    The ID of the gift card from which the specified amount was transferred.
    """

    amount_money: Money = pydantic.Field()
    """
    The amount added to the gift card balance for the transfer. This value is a positive integer.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .order import OrderParams
import typing
from .error import ErrorParams


class CalculateOrderResponseParams(typing_extensions.TypedDict):
    order: typing_extensions.NotRequired[OrderParams]
    """
    The calculated version of the order provided in the request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing


class CatalogQuerySetParams(typing_extensions.TypedDict):
    """
    The query filter to return the search result(s) by exact match of the specified `attribute_name` and any of
    the `attribute_values`.
    """

    attribute_name: str
    """
    The name of the attribute to be searched. Matching of the attribute name is exact.
    """

    attribute_values: typing.Sequence[str]
    """
    The desired values of the search attribute. Matching of the attribute values is exact and case insensitive.
    A maximum of 250 values may be searched in a request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing


class CatalogQueryTextParams(typing_extensions.TypedDict):
    """
    The query filter to return the search result whose searchable attribute values contain all of the specified keywords or tokens, independent of the token order or case.
    """

    keywords: typing.Sequence[str]
    """
    A list of 1, 2, or 3 search keywords. Keywords with fewer than 3 alphanumeric characters are ignored.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .bulk_upsert_merchant_custom_attributes_response_merchant_custom_attribute_upsert_response import (
    BulkUpsertMerchantCustomAttributesResponseMerchantCustomAttributeUpsertResponseParams,
)
from .error import ErrorParams


class BulkUpsertMerchantCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a [BulkUpsertMerchantCustomAttributes](api-endpoint:MerchantCustomAttributes-BulkUpsertMerchantCustomAttributes) response,
    which contains a map of responses that each corresponds to an individual upsert request.
    """

    values: typing_extensions.NotRequired[
        typing.Dict[str, BulkUpsertMerchantCustomAttributesResponseMerchantCustomAttributeUpsertResponseParams]
    ]
    """
    A map of responses that correspond to individual upsert requests. Each response has the
    same ID as the corresponding request and contains either a `merchant_id` and `custom_attribute` or an `errors` field.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawGroupsClient
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.customer_group import CustomerGroup
from ...types.list_customer_groups_response import ListCustomerGroupsResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.customer_group import CustomerGroupParams
from ...types.create_customer_group_response import CreateCustomerGroupResponse
from ...types.get_customer_group_response import GetCustomerGroupResponse
from ...types.update_customer_group_response import UpdateCustomerGroupResponse
from ...types.delete_customer_group_response import DeleteCustomerGroupResponse
from ...types.add_group_to_customer_response import AddGroupToCustomerResponse
from ...types.remove_group_from_customer_response import RemoveGroupFromCustomerResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawGroupsClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomerGroup]:
        """
        Retrieves the list of customer groups of a business.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the limit is less than 1 or greater than 50, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 50.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomerGroup]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.customers.groups.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/groups",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerGroupsResponse,
                    construct_type(
                        type_=ListCustomerGroupsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.groups
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        group: CustomerGroupParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerGroupResponse:
        """
        Creates a new customer group for a business.

        The request must include the `name` value of the group.

        Parameters
        ----------
        group : CustomerGroupParams
            The customer group to create.

        idempotency_key : typing.Optional[str]
            The idempotency key for the request. For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerGroupResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.create(
            group={"name": "Loyal Customers"},
        )
        """
        response = self._raw_client.create(
            group=group, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def get(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerGroupResponse:
        """
        Retrieves a specific customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerGroupResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.get(
            group_id="group_id",
        )
        """
        response = self._raw_client.get(group_id, request_options=request_options)
        return response.data

    def update(
        self, group_id: str, *, group: CustomerGroupParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateCustomerGroupResponse:
        """
        Updates a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to update.

        group : CustomerGroupParams
            The `CustomerGroup` object including all the updates you want to make.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerGroupResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.update(
            group_id="group_id",
            group={"name": "Loyal Customers"},
        )
        """
        response = self._raw_client.update(group_id, group=group, request_options=request_options)
        return response.data

    def delete(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerGroupResponse:
        """
        Deletes a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerGroupResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.delete(
            group_id="group_id",
        )
        """
        response = self._raw_client.delete(group_id, request_options=request_options)
        return response.data

    def add(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AddGroupToCustomerResponse:
        """
        Adds a group membership to a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to add to a group.

        group_id : str
            The ID of the customer group to add the customer to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddGroupToCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.add(
            customer_id="customer_id",
            group_id="group_id",
        )
        """
        response = self._raw_client.add(customer_id, group_id, request_options=request_options)
        return response.data

    def remove(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveGroupFromCustomerResponse:
        """
        Removes a group membership from a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to remove from the group.

        group_id : str
            The ID of the customer group to remove the customer from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveGroupFromCustomerResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.customers.groups.remove(
            customer_id="customer_id",
            group_id="group_id",
        )
        """
        response = self._raw_client.remove(customer_id, group_id, request_options=request_options)
        return response.data


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomerGroup]:
        """
        Retrieves the list of customer groups of a business.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single page. This limit is advisory. The response might contain more or fewer results.
            If the limit is less than 1 or greater than 50, Square returns a `400 VALUE_TOO_LOW` or `400 VALUE_TOO_HIGH` error. The default value is 50.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomerGroup]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.customers.groups.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/customers/groups",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListCustomerGroupsResponse,
                    construct_type(
                        type_=ListCustomerGroupsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.groups
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        group: CustomerGroupParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCustomerGroupResponse:
        """
        Creates a new customer group for a business.

        The request must include the `name` value of the group.

        Parameters
        ----------
        group : CustomerGroupParams
            The customer group to create.

        idempotency_key : typing.Optional[str]
            The idempotency key for the request. For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCustomerGroupResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.create(
                group={"name": "Loyal Customers"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            group=group, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def get(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomerGroupResponse:
        """
        Retrieves a specific customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomerGroupResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.get(
                group_id="group_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(group_id, request_options=request_options)
        return response.data

    async def update(
        self, group_id: str, *, group: CustomerGroupParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateCustomerGroupResponse:
        """
        Updates a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to update.

        group : CustomerGroupParams
            The `CustomerGroup` object including all the updates you want to make.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCustomerGroupResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.update(
                group_id="group_id",
                group={"name": "Loyal Customers"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(group_id, group=group, request_options=request_options)
        return response.data

    async def delete(
        self, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCustomerGroupResponse:
        """
        Deletes a customer group as identified by the `group_id` value.

        Parameters
        ----------
        group_id : str
            The ID of the customer group to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCustomerGroupResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.delete(
                group_id="group_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(group_id, request_options=request_options)
        return response.data

    async def add(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AddGroupToCustomerResponse:
        """
        Adds a group membership to a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to add to a group.

        group_id : str
            The ID of the customer group to add the customer to.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddGroupToCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.add(
                customer_id="customer_id",
                group_id="group_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.add(customer_id, group_id, request_options=request_options)
        return response.data

    async def remove(
        self, customer_id: str, group_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RemoveGroupFromCustomerResponse:
        """
        Removes a group membership from a customer.

        The customer is identified by the `customer_id` value
        and the customer group is identified by the `group_id` value.

        Parameters
        ----------
        customer_id : str
            The ID of the customer to remove from the group.

        group_id : str
            The ID of the customer group to remove the customer from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RemoveGroupFromCustomerResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.customers.groups.remove(
                customer_id="customer_id",
                group_id="group_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.remove(customer_id, group_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .scheduled_shift_filter import ScheduledShiftFilterParams
from .scheduled_shift_sort import ScheduledShiftSortParams


class ScheduledShiftQueryParams(typing_extensions.TypedDict):
    """
    Represents filter and sort criteria for the `query` field in a
    [SearchScheduledShifts](api-endpoint:Labor-SearchScheduledShifts) request.
    """

    filter: typing_extensions.NotRequired[ScheduledShiftFilterParams]
    """
    Filtering options for the query.
    """

    sort: typing_extensions.NotRequired[ScheduledShiftSortParams]
    """
    Sorting options for the query.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .timecard_sort_field import TimecardSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TimecardSort(UncheckedBaseModel):
    """
    Sets the sort order of search results.
    """

    field: typing.Optional[TimecardSortField] = pydantic.Field(default=None)
    """
    The field to sort on.
    See [TimecardSortField](#type-timecardsortfield) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    The order in which results are returned. Defaults to DESC.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .standard_unit_description import StandardUnitDescription
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class StandardUnitDescriptionGroup(UncheckedBaseModel):
    """
    Group of standard measurement units.
    """

    standard_unit_descriptions: typing.Optional[typing.List[StandardUnitDescription]] = pydantic.Field(default=None)
    """
    List of standard (non-custom) measurement units in this description group.
    """

    language_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    IETF language tag.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeleteCatalogObjectResponse(UncheckedBaseModel):
    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    deleted_object_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of all catalog objects deleted by this request.
    Multiple IDs may be returned when associated objects are also deleted, for example
    a catalog item variation will be deleted (and its ID included in this field)
    when its parent catalog item is deleted.
    """

    deleted_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The database [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates)
    of this deletion in RFC 3339 format, e.g., `2016-09-04T23:59:33.123Z`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class EmployeeWage(UncheckedBaseModel):
    """
    The hourly wage rate that an employee earns on a `Shift` for doing the job specified by the `title` property of this object. Deprecated at version 2020-08-26. Use [TeamMemberWage](entity:TeamMemberWage).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    employee_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `Employee` that this wage is assigned to.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    The job title that this wage relates to.
    """

    hourly_rate: typing.Optional[Money] = pydantic.Field(default=None)
    """
    Can be a custom-set hourly wage or the calculated effective hourly
    wage based on the annual wage and hours worked per week.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

Product = typing.Union[
    typing.Literal[
        "SQUARE_POS",
        "EXTERNAL_API",
        "BILLING",
        "APPOINTMENTS",
        "INVOICES",
        "ONLINE_STORE",
        "PAYROLL",
        "DASHBOARD",
        "ITEM_LIBRARY_IMPORT",
        "OTHER",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyAccountMapping(UncheckedBaseModel):
    """
    Represents the mapping that associates a loyalty account with a buyer.

    Currently, a loyalty account can only be mapped to a buyer by phone number. For more information, see
    [Loyalty Overview](https://developer.squareup.com/docs/loyalty/overview).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-assigned ID of the mapping.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the mapping was created, in RFC 3339 format.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number of the buyer, in E.164 format. For example, "+14155551111".
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class OrderRoundingAdjustmentParams(typing_extensions.TypedDict):
    """
    A rounding adjustment of the money being returned. Commonly used to apply cash rounding
    when the minimum unit of the account is smaller than the lowest physical denomination of the currency.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the rounding adjustment only within this order.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the rounding adjustment from the original sale order.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The actual rounding adjustment amount.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .loyalty_event_loyalty_account_filter import LoyaltyEventLoyaltyAccountFilter
import pydantic
from .loyalty_event_type_filter import LoyaltyEventTypeFilter
from .loyalty_event_date_time_filter import LoyaltyEventDateTimeFilter
from .loyalty_event_location_filter import LoyaltyEventLocationFilter
from .loyalty_event_order_filter import LoyaltyEventOrderFilter
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyEventFilter(UncheckedBaseModel):
    """
    The filtering criteria. If the request specifies multiple filters,
    the endpoint uses a logical AND to evaluate them.
    """

    loyalty_account_filter: typing.Optional[LoyaltyEventLoyaltyAccountFilter] = pydantic.Field(default=None)
    """
    Filter events by loyalty account.
    """

    type_filter: typing.Optional[LoyaltyEventTypeFilter] = pydantic.Field(default=None)
    """
    Filter events by event type.
    """

    date_time_filter: typing.Optional[LoyaltyEventDateTimeFilter] = pydantic.Field(default=None)
    """
    Filter events by date time range. 
    For each range, the start time is inclusive and the end time 
    is exclusive.
    """

    location_filter: typing.Optional[LoyaltyEventLocationFilter] = pydantic.Field(default=None)
    """
    Filter events by location.
    """

    order_filter: typing.Optional[LoyaltyEventOrderFilter] = pydantic.Field(default=None)
    """
    Filter events by the order associated with the event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .checkout_location_settings_policy import CheckoutLocationSettingsPolicy
from .checkout_location_settings_branding import CheckoutLocationSettingsBranding
from .checkout_location_settings_tipping import CheckoutLocationSettingsTipping
from .checkout_location_settings_coupons import CheckoutLocationSettingsCoupons
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutLocationSettings(UncheckedBaseModel):
    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the location that these settings apply to.
    """

    customer_notes_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether customers are allowed to leave notes at checkout.
    """

    policies: typing.Optional[typing.List[CheckoutLocationSettingsPolicy]] = pydantic.Field(default=None)
    """
    Policy information is displayed at the bottom of the checkout pages.
    You can set a maximum of two policies.
    """

    branding: typing.Optional[CheckoutLocationSettingsBranding] = pydantic.Field(default=None)
    """
    The branding settings for this location.
    """

    tipping: typing.Optional[CheckoutLocationSettingsTipping] = pydantic.Field(default=None)
    """
    The tip settings for this location.
    """

    coupons: typing.Optional[CheckoutLocationSettingsCoupons] = pydantic.Field(default=None)
    """
    The coupon settings for this location.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the settings were last updated, in RFC 3339 format.
    Examples for January 25th, 2020 6:25:34pm Pacific Standard Time:
    UTC: 2020-01-26T02:25:34Z
    Pacific Standard Time with UTC offset: 2020-01-25T18:25:34-08:00
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .device_code import DeviceCodeParams


class ListDeviceCodesResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    device_codes: typing_extensions.NotRequired[typing.Sequence[DeviceCodeParams]]
    """
    The queried DeviceCode.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    A pagination cursor to retrieve the next set of results for your
    original query to the endpoint. This value is present only if the request
    succeeded and additional results are available.
    
    See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .booking_custom_attribute_upsert_response import BookingCustomAttributeUpsertResponseParams
from .error import ErrorParams


class BulkUpsertBookingCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a [BulkUpsertBookingCustomAttributes](api-endpoint:BookingCustomAttributes-BulkUpsertBookingCustomAttributes) response,
    which contains a map of responses that each corresponds to an individual upsert request.
    """

    values: typing_extensions.NotRequired[typing.Dict[str, BookingCustomAttributeUpsertResponseParams]]
    """
    A map of responses that correspond to individual upsert requests. Each response has the
    same ID as the corresponding request and contains either a `booking_id` and `custom_attribute` or an `errors` field.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """
